package collector_go

import (
	"encoding/json"
	"errors"
	"fmt"
	"io/ioutil"
	s "loafle.com/overflow/collector_go/scheduler"
	conf "loafle.com/overflow/crawler_go/config"
	"log"
	"os"
	"path/filepath"
	"strings"
	"time"
)

const CONFIG_ROOT = "/config/container/"

type Collector struct {
	scheduler s.Scheduler
	configs   []*conf.Config
}

func (c *Collector) Start() {
	go func() {
		c.configs = make([]*conf.Config, 0)

		if err := c.readAllConfig(CONFIG_ROOT); err != nil {
			log.Println(err)
		}

		c.scheduler = s.Scheduler{}
		c.scheduler.Init()

		for _, conf := range c.configs {
			if err := c.addSensor(conf); err != nil {
				log.Println(err)
			}
		}
	}()
}

func (c *Collector) AddSensor(container, crawler, id string) {
	config := c.readConfig(CONFIG_ROOT + container + "/" + crawler + "/" + id + ".conf")
	if config != nil {
		if err := c.addSensor(config); err != nil {
			log.Println(err)
		}
	}
}

func (c *Collector) RemoveSensor(id string) {
	if err := c.scheduler.RemoveSchedule(id); err != nil {
		log.Println(err)
	}
}

func (c *Collector) UpdateSensor(id, interval string) {
	if !c.checkExist(id) {
		log.Println("Cannot update Sensor : ID not exist [" + id + "]")
		return
	}
	c.scheduler.UpdateSchedule(id, interval)
}

func (c *Collector) Stop() {
	c.scheduler.RemoveAllSchedule()
	c.scheduler.Stop()
}

func (c *Collector) collect(id string) {
	log.Printf("COLLECT %s - [ID: %s]", time.Now(), id)
}

func (c *Collector) addSensor(conf *conf.Config) error {
	return c.scheduler.NewSchedule(conf.Id, conf.Schedule.Interval, c.collect)
}

func (c *Collector) readAllConfig(rootPath string) error {

	err := filepath.Walk(rootPath, func(path string, f os.FileInfo, err error) error {
		if err != nil {
			return err
		}
		if !f.IsDir() && strings.HasSuffix(f.Name(), ".conf") {
			c.readConfig(path)
		}
		return nil
	})

	if err != nil {
		return err
	}

	if len(c.configs) <= 0 {
		return errors.New("No configuration file found.")
	}

	return nil
}

func (c *Collector) readConfig(path string) *conf.Config {
	bytes, err := ioutil.ReadFile(path)
	if err != nil {
		fmt.Println(err)
	}
	conf := conf.Config{}
	json.Unmarshal(bytes, &conf)

	if err := c.validateConfig(&conf, path); err != nil {
		fmt.Println(err)
		return nil
	}
	c.configs = append(c.configs, &conf)
	return &conf
}

func (c *Collector) validateConfig(conf *conf.Config, configPath string) error {

	//todo : some validations
	if c.checkExist(conf.Id) {
		return errors.New("The Same Id already exists. " + configPath)
	}
	return nil
}

func (c *Collector) checkExist(id string) bool {
	for _, c := range c.configs {
		if c.Id == id {
			return true
		}
	}
	return false
}

// connection
//func CallGet() {
//
//	conn, err := grpc.Dial(address, grpc.WithInsecure())
//	if err != nil {
//		log.Fatalf("did not connect: %v", err)
//	}
//	defer conn.Close()
//
//
//	dc := g.NewDataClient(conn);
//
//	in := &g.Input{}
//
//	in.Id = ""
//	in.Name = g.Crawlers_HEALTH_DNS
//
//
//
//	out, err := dc.Get(context.Background(), in) ////
//
//	if err != nil {
//		log.Println(err)
//	}
//	log.Println(out)
//
//}
